#!/usr/bin/env python3
import os
import os.path
import json
import sys
import http.client
import json

VALID_MODELS = [
    'gpt-4',
    'gpt-4-32k',
    'gpt-3.5-turbo',
]

# Use an environment variable, or replace the following line with your OpenAI API key
OPENAI_API_KEY = os.getenv('OPENAI_API_KEY')

if not OPENAI_API_KEY:
    print('Missing environment variable OPENAI_API_KEY. It is required to use this script.')
    sys.exit(1)

def usage():
    print(f'USAGE: python3 {sys.argv[0]} [args]')
    print(f'')
    print(f'If there are additional arguments, they are sent as the system prompt.')
    print(f'Otherwise the prompt: "You are here to help." is sent')
    print(f'')
    print(f'The user prompt is read from standard input.')
    print(f'')
    print(f'args:')
    print(f'')
    print(f'    -help  --help: show this help text')
    for model in VALID_MODELS:
        print(f'    --{model}: use this version of ChatGPT')

def main():
    model = VALID_MODELS[0]
    system_prompt = None
    user_prompt = None
    extra_args = []
    for arg in sys.argv[1:]:
        if arg == '-h' or arg == '--help':
            usage()
            return
        if arg in (f'--{m}' for m in VALID_MODELS):
            model = arg[2:]
        else:
            extra_args.append(arg)
    if len(extra_args) > 0:
        system_prompt = ' '.join(extra_args)
    try:
        additional_input = sys.stdin.read()
        if additional_input.strip():
            user_prompt = additional_input
    except:
        return

    if system_prompt is None:
        system_prompt = 'You are here to help.'

    messages = [{'role': 'system', 'content': system_prompt }]
    if user_prompt is not None:
        messages.append({'role': 'user', 'content': user_prompt})

    conn = http.client.HTTPSConnection("api.openai.com")
    headers = {
        'Content-Type': 'application/json',
        'Authorization': f'Bearer {OPENAI_API_KEY}',
    }

    conn.request("POST", f'/v1/chat/completions', body=json.dumps({
        'model': model,
        'messages': messages,
    }), headers=headers)

    res = conn.getresponse()
    data = res.read()
    response = json.loads(data.decode('utf-8'))

    if 'choices' in response and len(response['choices']) > 0 and response['choices'][0]['finish_reason'] == 'stop':
        print(response['choices'][0]['message']['content'])
    else:
        print('Error, unexpected response:')
        print(response)

main()
